package vip.xiaonuo.money.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.beans.factory.annotation.Autowired;
import vip.xiaonuo.money.entity.MoneyBusiness;
import vip.xiaonuo.money.mapper.MoneyBusinessMapper;
import vip.xiaonuo.money.param.MoneyBusinessPageParam;
import vip.xiaonuo.money.param.MoneyBusinessParam;
import vip.xiaonuo.money.service.IMoneyBusinessService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 财务营业情况 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-11-19
 */
@Service
public class MoneyBusinessServiceImpl extends ServiceImpl<MoneyBusinessMapper, MoneyBusiness> implements IMoneyBusinessService {

    @Autowired
    private MoneyBusinessMapper businessMapper;

    @Override
    public JSONArray listBusiness(MoneyBusinessParam moneyBusinessParam) {
        String orgId = moneyBusinessParam.getOrgId();
        String time = moneyBusinessParam.getTime();//2024-10
        QueryWrapper<MoneyBusiness> queryWrapper = new QueryWrapper<>();
        QueryWrapper<MoneyBusiness> queryWrapper1 = new QueryWrapper<>();
        if ("".equals(orgId) || null == orgId) {
            queryWrapper.eq("type", 1);
            queryWrapper1.eq("type", 1);
        } else {
            queryWrapper.eq("org_id", orgId).eq("type", 0);
            queryWrapper1.eq("org_id", orgId).eq("type", 0);
        }
        JSONArray jsonArray = new JSONArray();
        if ("".equals(time) || null == time) {
            DateTime dateTime = DateUtil.beginOfMonth(new Date());
            queryWrapper.ge("create_time", dateTime);
            DateTime beginOfYear = DateUtil.beginOfYear(new Date());
            queryWrapper1.ge("create_time", beginOfYear);
        } else {
            String year = time.substring(0, 4);
            queryWrapper.between("create_time", time + "-01 00:00:00", time + "-31 23:59:59");
            queryWrapper1.between("create_time", year + "-01-01 00:00:00", time + "-31 23:59:59");
        }
        DecimalFormat def = new DecimalFormat("0.00");
        MoneyBusiness moneyBusiness = new MoneyBusiness();
        Double value1 = 0.00;
        Double value2 = 0.00;
        Double value3 = 0.00;
        Double value4 = 0.00;
        Double value5 = 0.00;
        Double value6 = 0.00;
        Double value7 = 0.00;
        Double value8 = 0.00;
        Double value9 = 0.00;
        Double value10 = 0.00;
        List<MoneyBusiness> businesses = businessMapper.selectList(queryWrapper);
        List<MoneyBusiness> businessList = businessMapper.selectList(queryWrapper1);
        if (businesses.size() != 0) {
            for (MoneyBusiness business : businesses) {
                value1 = value1 + Double.valueOf(business.getBusinessIncome());
                value2 = value2 + Double.valueOf(business.getBusinessCost());
                value3 = value3 + Double.valueOf(business.getProfit());
                value4 = value4 + Double.valueOf(business.getTaxes());
                value5 = value5 + Double.valueOf(business.getBalance());
            }
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("type", "month");
        jsonObject.put("object", moneyBusiness);
        moneyBusiness.setBusinessIncome(def.format(value1));
        moneyBusiness.setBusinessCost(def.format(value2));
        moneyBusiness.setProfit(def.format(value3));
        moneyBusiness.setTaxes(def.format(value4));
        moneyBusiness.setBalance(def.format(value5));
        if (businessList.size() != 0) {
            for (MoneyBusiness business : businessList) {
                value6 = value6 + Double.valueOf(business.getBusinessIncome());
                value7 = value7 + Double.valueOf(business.getBusinessCost());
                value8 = value8 + Double.valueOf(business.getProfit());
                value9 = value9 + Double.valueOf(business.getTaxes());
                value10 = value10 + Double.valueOf(business.getBalance());
            }
        }
        JSONObject jsonObject1 = new JSONObject();
        MoneyBusiness moneyBusiness1 = new MoneyBusiness();
        moneyBusiness1.setBusinessIncome(def.format(value6));
        moneyBusiness1.setBusinessCost(def.format(value7));
        moneyBusiness1.setProfit(def.format(value8));
        moneyBusiness1.setTaxes(def.format(value9));
        moneyBusiness1.setBalance(def.format(value10));
        jsonObject1.put("type", "year");
        jsonObject1.put("object", moneyBusiness1);
        jsonArray.add(jsonObject);
        jsonArray.add(jsonObject1);
        return jsonArray;
    }

    @Override
    public Page<MoneyBusiness> getPage(MoneyBusinessPageParam pageParam) {
        String orgId = pageParam.getOrgId();
        Page<MoneyBusiness> page = new Page(pageParam.getPageNum(), pageParam.getPageSize());
        QueryWrapper<MoneyBusiness> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        return businessMapper.selectPage(page, queryWrapper);
    }

    @Override
    public void addBusiness(MoneyBusiness business) {
        business.setCreateTime(new Date());
        businessMapper.insert(business);
    }

    @Override
    public void updateBusiness(MoneyBusiness business) {
        businessMapper.updateById(business);
    }

    @Override
    public void deleteBusiness(MoneyBusiness business) {
        businessMapper.deleteById(business);
    }
}
