package vip.xiaonuo.purchase.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import vip.xiaonuo.purchase.entity.PurchaseOrder;
import vip.xiaonuo.purchase.entity.vo.*;
import vip.xiaonuo.purchase.mapper.PurchaseOrderMapper;
import vip.xiaonuo.purchase.param.PurchaseOrderParam;
import vip.xiaonuo.purchase.param.PurchaseParam;
import vip.xiaonuo.purchase.service.IPurchaseOrderService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 采购订单表 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-11-25
 */
@Service
public class PurchaseOrderServiceImpl extends ServiceImpl<PurchaseOrderMapper, PurchaseOrder> implements IPurchaseOrderService {

    @Autowired
    private PurchaseOrderMapper purchaseOrderMapper;

    @Override
    public PurchaseOrderVo getOrder(PurchaseParam purchaseParam) {
        String orgId = purchaseParam.getOrgId();
        QueryWrapper<PurchaseOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId).orderByAsc("create_time");
        String time = purchaseParam.getTime();
        DateTime dateTime = DateUtil.beginOfYear(new Date());
        if ("".equals(time) || null == time) {
            queryWrapper.ge("create_time", dateTime);
        } else {
            queryWrapper.between("create_time", dateTime, time);
        }
        List<PurchaseOrder> purchaseOrders = purchaseOrderMapper.selectList(queryWrapper);
        PurchaseOrderVo vo = new PurchaseOrderVo();
        double money = 0.00;
        double acceptNumber = 0.00;
        double acceptMoney = 0.00;
        double enterNumber = 0.00;
        double enterMoney = 0.00;
        double invoiceMoney = 0.00;
        int invoiceNumber = 0;
        DecimalFormat def = new DecimalFormat("0.00");
        if (purchaseOrders.size() != 0) {
            for (PurchaseOrder purchaseOrder : purchaseOrders) {
                money = money + Double.valueOf(purchaseOrder.getPurchaseMoney());
                acceptMoney = acceptMoney + Double.valueOf(purchaseOrder.getMaterialAcceptPrice());
                acceptNumber = acceptNumber + Double.valueOf(purchaseOrder.getMaterialAcceptNumber());
                enterNumber = enterNumber + Double.valueOf(purchaseOrder.getEnterNumber());
                enterMoney = enterMoney + Double.valueOf(purchaseOrder.getEnterPrice());
                invoiceMoney = invoiceMoney + Double.valueOf(purchaseOrder.getInvoicePrice());
                invoiceNumber = invoiceNumber + purchaseOrder.getInvoiceNumber();
            }
            vo.setMonthRank(purchaseOrders.get(purchaseOrders.size() - 1).getMonthRank());
            vo.setRank(purchaseOrders.get(purchaseOrders.size() - 1).getRank());
            vo.setOrderNumber(String.valueOf(purchaseOrders.size()));
        } else {
            vo.setMonthRank("0");
            vo.setRank("0");
            vo.setOrderNumber("0");
        }
        vo.setPurchaseMoney(def.format(money));
        vo.setMaterialAcceptPrice(def.format(acceptMoney));
        vo.setMaterialAcceptNumber(def.format(acceptNumber));
        vo.setEnterNumber(def.format(enterNumber));
        vo.setEnterPrice(def.format(enterMoney));
        vo.setInvoiceNumber(invoiceNumber);
        vo.setInvoicePrice(def.format(invoiceMoney));

        LocalDate currentDate = LocalDate.now();
        int lastYear = currentDate.getYear() - 1;
        QueryWrapper<PurchaseOrder> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq(StringUtils.isNotBlank(orgId), "org_id", orgId).orderByAsc("create_time");
        queryWrapper1.between("create_time", lastYear + "-01-01 00:00:00", lastYear + "-12-31 23:59:59");
        queryWrapper1.select(" CONVERT(IFNULL(avg(purchase_money),'0'),decimal(10,2)) 'purchaseMoney'");
        PurchaseOrder purchaseOrder = purchaseOrderMapper.selectOne(queryWrapper1);
        if (ObjectUtil.isNotEmpty(purchaseOrder)) {
            vo.setLastAvgPrice(purchaseOrder.getPurchaseMoney());
        } else {
            vo.setLastAvgPrice("0.00");
        }
        return vo;
    }

    @Override
    public List<PurchaseOrderMoneyVo> listOrderMoney(PurchaseOrderParam orderParam) {
        String orgId = orderParam.getOrgId();
        QueryWrapper<PurchaseOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId).orderByAsc("create_time");
        queryWrapper.select(" IFNULL(sum(purchase_money),'0') purchaseMoney,create_time createTime,count(*) invoiceNumber");

        List<PurchaseOrderMoneyVo> list = new ArrayList<>();
        if ("".equals(orderParam.getTimeType()) || null == orderParam.getTimeType()||"0".equals(orderParam.getTimeType())) {
            if ("".equals(orderParam.getOrderType()) || null == orderParam.getOrderType()) {
                queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y')");
            } else {
                queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y')" + "," + orderParam.getOrderType());
            }

            List<PurchaseOrder> orders = purchaseOrderMapper.selectList(queryWrapper);
            if (orders.size() != 0) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
                for (PurchaseOrder order : orders) {
                    PurchaseOrderMoneyVo vo = new PurchaseOrderMoneyVo();
                    vo.setTime(sdf.format(order.getCreateTime()));
                    vo.setOrderMoney(order.getPurchaseMoney());
                    vo.setOrderNumber(order.getInvoiceNumber());
                    list.add(vo);
                }
            }
        } else {
            DateTime dateTime = DateUtil.beginOfYear(new Date());
            queryWrapper.ge("create_time", dateTime);
            if ("".equals(orderParam.getOrderType()) || null == orderParam.getOrderType()) {
                queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')");
            } else {
                queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')" + "," + orderParam.getOrderType());
            }
            List<PurchaseOrder> orders = purchaseOrderMapper.selectList(queryWrapper);
            if (orders.size() != 0) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
                for (PurchaseOrder order : orders) {
                    PurchaseOrderMoneyVo vo = new PurchaseOrderMoneyVo();
                    vo.setTime(sdf.format(order.getCreateTime()));
                    vo.setOrderMoney(order.getPurchaseMoney());
                    vo.setOrderNumber(order.getInvoiceNumber());
                    list.add(vo);
                }
            }
        }
        return list;
    }

    @Override
    public List<PurchaseOrderTypeVo> orderType(PurchaseParam param) {
        String orgId = param.getOrgId();
        DateTime month = DateUtil.beginOfMonth(new Date());
        DateTime year = DateUtil.beginOfYear(new Date());
        LocalDate currentDate = LocalDate.now();
        int lastYear = currentDate.getYear() - 1;
        LocalDate lastYearSameDate = currentDate.minus(1, ChronoUnit.YEARS); // 减去一年
        int lastYearSameMonth = lastYearSameDate.getMonthValue(); // 获取去年同月份
        String lastMonth = "";
        if (lastYearSameMonth > 9) {
            lastMonth = String.valueOf(lastYearSameMonth);
        } else {
            lastMonth = "0" + lastYearSameMonth;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<PurchaseOrderTypeVo> orderTypeVos = purchaseOrderMapper.listType(sdf.format(month), sdf.format(year), lastYear + "-" + lastMonth, lastYear, orgId);
        return orderTypeVos;
    }

    @Override
    public List<PurchaseMoneyVo> listMoney(PurchaseParam param) {
        QueryWrapper<PurchaseOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(param.getOrgId()), "org_id", param.getOrgId());
        DateTime year = DateUtil.beginOfYear(new Date());
        queryWrapper.ge("create_time", year);
        queryWrapper.select(" IFNULL(sum(purchase_money),'0') purchaseMoney,create_time createTime");
        queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')");
        List<PurchaseOrder> orders = purchaseOrderMapper.selectList(queryWrapper);
        queryWrapper.clear();
        queryWrapper.eq(StringUtils.isNotBlank(param.getOrgId()), "org_id", param.getOrgId());
        LocalDate currentDate = LocalDate.now();
        int lastYear = currentDate.getYear() - 1;
        queryWrapper.between("create_time", lastYear + "-01-01 00:00:00", lastYear + "-12-31 23:59:59");
        queryWrapper.select(" IFNULL(sum(purchase_money),'0') purchaseMoney,create_time createTime");
        queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')");
        List<PurchaseOrder> orders1 = purchaseOrderMapper.selectList(queryWrapper);
        List<PurchaseMoneyVo> list = new ArrayList<>();

        for (int i = 1; i < 13; i++) {
            PurchaseMoneyVo vo = new PurchaseMoneyVo();
            vo.setTime(i + "月");
            for (PurchaseOrder order : orders) {
                Calendar cal = Calendar.getInstance();
                cal.setTime(order.getCreateTime());
                int month = cal.get(Calendar.MONTH) + 1; // 注意月份是从0开始计数的，所以需要加1
                if (i == month) {
                    vo.setMoney(order.getPurchaseMoney());
                }
            }
            for (PurchaseOrder order : orders1) {
                Calendar cal = Calendar.getInstance();
                cal.setTime(order.getCreateTime());
                int month = cal.get(Calendar.MONTH) + 1; // 注意月份是从0开始计数的，所以需要加1
                if (i == month) {
                    vo.setLastMoney(order.getPurchaseMoney());
                }
            }
            list.add(vo);
        }
        return list;
    }

    @Override
    public List<PurchaseMaterialVo> materialType(PurchaseParam param) {
        QueryWrapper<PurchaseOrder> queryWrapper = new QueryWrapper<>();
        String time = param.getTime();
        DateTime dateTime = DateUtil.beginOfYear(new Date());
        if ("".equals(time) || null == time) {
            queryWrapper.ge("create_time", dateTime);
        } else {
            queryWrapper.between("create_time", time+"-01 00:00:00", time+"-31 23:59:59");
        }
        queryWrapper.eq(StringUtils.isNotBlank(param.getOrgId()), "org_id", param.getOrgId());
        List<PurchaseMaterialVo> list = purchaseOrderMapper.getSupplier(queryWrapper);
        return list;
    }

}
