package vip.xiaonuo.warehouse.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import vip.xiaonuo.warehouse.entity.*;

import vip.xiaonuo.warehouse.entity.vo.WarehouseStockSecondVo;
import vip.xiaonuo.warehouse.mapper.WarehouseIncomStockMapper;
import vip.xiaonuo.warehouse.mapper.WarehouseOutStockMapper;
import vip.xiaonuo.warehouse.mapper.WarehouseStockMapper;
import vip.xiaonuo.warehouse.mapper.WarehouseStockSecondMapper;
import vip.xiaonuo.warehouse.param.WarehouseStockParam;
import vip.xiaonuo.warehouse.param.WarehouseStockSecondParam;
import vip.xiaonuo.warehouse.service.IWarehouseStockService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 仓库库存变化 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-11-28
 */
@Service
public class WarehouseStockServiceImpl extends ServiceImpl<WarehouseStockMapper, WarehouseStock> implements IWarehouseStockService {

    @Autowired
    private WarehouseStockMapper stockMapper;
    @Autowired
    private WarehouseIncomStockMapper incomStockMapper;
    @Autowired
    private WarehouseOutStockMapper outStockMapper;
    @Autowired
    private WarehouseStockSecondMapper secondMapper;

    @Override
    public WarehouseStockVo getOneTail(WarehouseStockParam stockParam) {
        String orgId = stockParam.getOrgId();
        QueryWrapper<WarehouseStock> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.orderByDesc("create_time").last(" limit 1");
        WarehouseStock warehouseStock = stockMapper.selectOne(queryWrapper);
        DateTime day = DateUtil.beginOfDay(new Date());
        QueryWrapper<WarehouseIncomStock> incomQueryWrapper = new QueryWrapper<>();
        incomQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        incomQueryWrapper.ge("create_time", day);
        WarehouseIncomStock incomeStock = incomStockMapper.selectOne(incomQueryWrapper);
        QueryWrapper<WarehouseOutStock> outQueryWrapper = new QueryWrapper<>();
        outQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        outQueryWrapper.ge("create_time", day);
        WarehouseOutStock outStock = outStockMapper.selectOne(outQueryWrapper);
        WarehouseStockVo vo = new WarehouseStockVo();
        if (ObjectUtil.isNotEmpty(warehouseStock)) {
            vo.setLanolin(warehouseStock.getLanolin());
            vo.setFishOil(warehouseStock.getFishOil());
        }else {
            vo.setFishOil("0");
            vo.setLanolin("0");
        }
        if (ObjectUtil.isNotEmpty(incomeStock)) {
            vo.setMaterialIncome(incomeStock.getMaterial());
            vo.setFinishProduceIncome(incomeStock.getFinishProduce());
            vo.setHalfProduceIncome(incomeStock.getHalfProduce());
            vo.setLowValueIncome(incomeStock.getLowValue());
            vo.setPackageMaterialIncome(incomeStock.getPackageMaterial());
            vo.setLanolinIncome(incomeStock.getLanolin());
            vo.setFishOilIncome(incomeStock.getFishOil());
        }
        if (ObjectUtil.isNotEmpty(outStock)) {
            vo.setMaterialOut(outStock.getMaterial());
            vo.setFinishProduceOut(outStock.getFinishProduce());
            vo.setHalfProduceOut(outStock.getHalfProduce());
            vo.setLowValueOut(outStock.getLowValue());
            vo.setPackageMaterialOut(outStock.getPackageMaterial());
            vo.setLanolinOut(outStock.getLanolin());
            vo.setFishOilOut(outStock.getFishOil());
        }
        DateTime month = DateUtil.beginOfMonth(new Date());
        incomQueryWrapper.clear();
        incomQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        incomQueryWrapper.groupBy("lanolin_supplier");
        Long count = incomStockMapper.selectCount(incomQueryWrapper);
        incomQueryWrapper.clear();
        incomQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        incomQueryWrapper.groupBy("fish_oil_supplier");
        Long count1 = incomStockMapper.selectCount(incomQueryWrapper);
        if (null==count) {
            vo.setLanolinSupplier(0);
        } else {
            vo.setLanolinSupplier(Integer.parseInt(String.valueOf(count)));
        }
        if (null==count1) {
            vo.setLanolinSupplier(0);
        } else {
            vo.setFishOilSupplier(Integer.parseInt(String.valueOf(count1)));
        }


        incomQueryWrapper.clear();
        outQueryWrapper.clear();
        incomQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        incomQueryWrapper.select(" IFNULL(sum(lanolin),'0') lanolin,IFNULL(sum(fish_oil),'0') fishOil");
        incomQueryWrapper.ge("create_time", month);
        outQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        outQueryWrapper.select(" IFNULL(sum(lanolin),'0') lanolin,IFNULL(sum(fish_oil),'0') fishOil");
        outQueryWrapper.ge("create_time", month);
        WarehouseIncomStock stock = incomStockMapper.selectOne(incomQueryWrapper);
        WarehouseOutStock outStock1 = outStockMapper.selectOne(outQueryWrapper);
        if (ObjectUtil.isNotEmpty(stock)) {
            vo.setLanolinIncomeMonth(stock.getLanolin());
            vo.setFishOilIncomeMonth(stock.getFishOil());
        }
        if (ObjectUtil.isNotEmpty(outStock1)) {
            vo.setLanolinOutMonth(outStock1.getLanolin());
            vo.setFishOilOutMonth(outStock1.getFishOil());
        }
        return vo;
    }

    @Override
    public List<WarehouseStock> listStockChange(WarehouseStockParam stockParam) {
        String orgId = stockParam.getOrgId();
        QueryWrapper<WarehouseStock> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.select(" IFNULL(sum(material),'0') material,IFNULL(sum(finish_produce),'0') finishProduce" +
                ",IFNULL(sum(half_produce),'0') halfProduce,IFNULL(sum(low_value),'0') lowValue,IFNULL(sum(package_material),'0') packageMaterial,create_time createTime");
        queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')");
        queryWrapper.orderByAsc("create_time");
        return stockMapper.selectList(queryWrapper);
    }

    @Override
    public List<WarehouseOutStock> listStockOut(WarehouseStockParam stockParam) {
        String orgId = stockParam.getOrgId();
        QueryWrapper<WarehouseOutStock> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.select(" IFNULL(sum(material),'0') material,IFNULL(sum(finish_produce),'0') finishProduce" +
                ",IFNULL(sum(half_produce),'0') halfProduce,IFNULL(sum(low_value),'0') lowValue,IFNULL(sum(package_material),'0') packageMaterial,create_time createTime");
        queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')");
        queryWrapper.orderByAsc("create_time");
        return outStockMapper.selectList(queryWrapper);
    }

    @Override
    public WarehouseStockSecondVo getSecond(WarehouseStockSecondParam param) {
        String orgId = param.getOrgId();
        Integer materialName = param.getMaterialType();
        QueryWrapper<WarehouseStockSecond> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId),"org_id",orgId);
        queryWrapper.eq("material_type", materialName);
        DateTime month = DateUtil.beginOfMonth(new Date());
        queryWrapper.ge("create_time", month);
        queryWrapper.select(" IFNULL(sum(income_money),'0') incomeMoney,stock stock,IFNULL(sum(income_number),'0') incomeNumber");
        WarehouseStockSecond stockSecond = secondMapper.selectOne(queryWrapper);
        queryWrapper.clear();
        queryWrapper.eq(StringUtils.isNotBlank(orgId),"org_id",orgId);
        queryWrapper.eq("material_type", materialName);
        queryWrapper.select(" IFNULL(sum(income_money),'0') incomeMoney");
        WarehouseStockSecond second = secondMapper.selectOne(queryWrapper);
        WarehouseStockSecondVo vo = new WarehouseStockSecondVo();
        vo.setIncomeMoney(stockSecond.getIncomeMoney());
        vo.setIncomeNumber(stockSecond.getIncomeNumber());
        vo.setMoneyCount(second.getIncomeMoney());
        vo.setStock(stockSecond.getStock());
        return vo;
    }

}
